package com.unissoft.controller;

import com.unissoft.config.MyApplicationContextAware;
import com.unissoft.constant.GrainConstant;
import com.unissoft.dto.MessageVO;
import com.unissoft.enums.OnlineStatusEnum;
import com.unissoft.redismq.PublishService;
import com.unissoft.service.SysMessageService;
import com.unissoft.utils.JsonUtil;
import com.unissoft.utils.SpringUtils;
import com.unissoft.vo.UserOnlineStatusVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@ServerEndpoint("/webSocket/{userId}")
@Component
public class WebSocketServer {

    /** 用来存放每个客户端对应的WebSocket的Session对象 */
    private static ConcurrentHashMap<Integer, Session> sessionPools = new ConcurrentHashMap<>();

    /**
     * 建立连接成功调用
     * @param session   连接成功后，客户端session对象
     * @param userId    用户ID
     */
    @OnOpen
    public void onOpen(Session session, @PathParam(value = "userId") Integer userId) {
        if(sessionPools.containsKey(userId)) {
            sessionPools.remove(userId);
        }
        sessionPools.put(userId, session);
        addOnlineUser(userId);
        publishStatisticsOnlineStatus();//发布在线统计状态

        //登录后，查询是否有待推送的通知，如果有立马推送
        new Thread() {
            @Override
            public void run() {
                pushMessage2User(userId, session);
            }
        }.start();
    }

    private void pushMessage2User(Integer userId, Session session) {
        SysMessageService sysMessageService = MyApplicationContextAware.getBean(SysMessageService.class);

        List<MessageVO> list = sysMessageService.getUnpushedUserMessage(userId);
        if(null != list && list.isEmpty() == false) {
            for(MessageVO vo : list) {
                String content = JsonUtil.parseObjToJson(vo);
                sendMessage(session, content);
            }
            //设置为已推送状态
            sysMessageService.batchSetPushedStatus(list);
        }
    }

    private void publishStatisticsOnlineStatus() {
        PublishService publishService = MyApplicationContextAware.getBean(PublishService.class);
        UserOnlineStatusVO onlineStatusVO = new UserOnlineStatusVO();
        onlineStatusVO.setOnlineStatus(OnlineStatusEnum.STATISTICS.getCode());
        publishService.publish(GrainConstant.MQ_USER_STATUS_CHANNEL, onlineStatusVO);
    }


    /**
     * 关闭连接时调用
     * @param userId    用户ID
     */
    @OnClose
    public void onClose(@PathParam(value = "userId") Integer userId) {
        removeSession(userId);
        publishStatisticsOnlineStatus();//发布在线统计状态
    }

    public static void removeSession(Integer userId) {
        if(sessionPools.containsKey(userId)) {
            sessionPools.remove(userId);
        }
        subOnlineUser(userId);
    }

    /**
     * 服务端收到客户端信息后，向接收端发送 messageContent
     * @param message
     */
    @OnMessage
    public void onMessage(String message, @PathParam(value = "userId") Integer userId) {
        MessageVO messageVO = JsonUtil.parseJsonToObj(message, MessageVO.class);
        if(null != messageVO) {
            messageVO.setFromUserId(userId);
            SysMessageService sysMessageService = MyApplicationContextAware.getBean(SysMessageService.class);
            sysMessageService.sendMessage(messageVO);
        }
    }

    /**
     * 错误时调用
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("发生错误，错误原因：" + error.getMessage());
    }


    /**
     * 向指定用户发送信息
     * @param toUserId  接收的用户id
     * @param content   发送内容
     */
    public boolean sendUserMessage(Integer toUserId, String content) {
        Session session = sessionPools.get(toUserId);
        if(null == session) {//用户不在线，返回false
            return false;
        }
        return sendMessage(session, content);
    }

    public void noticeOnlineNum2All(long onlineNum) {
        MessageVO messageVO = new MessageVO();
        messageVO.setType(9999);//表示在线用户数信息
        messageVO.setOnlineNum(onlineNum);
        String content = JsonUtil.parseObjToJson(messageVO);
        for(Session toSession : sessionPools.values()) {
            sendMessage(toSession, content);
        }
    }


    /**
     * 向某客户端发送消息
     * @param toSession   接收客户端session
     * @param content   信息内容
     * @throws IOException
     */
    private synchronized boolean sendMessage(Session toSession, String content) {
        try {
            toSession.getBasicRemote().sendText(content);
            return true;
        } catch (IOException e) {//推送失败，返回false
            log.error(e.getMessage());
            return false;
        }
    }


    /**
     * 在线用户添加
     */
    public static void addOnlineUser(Integer userId) {
        RedisTemplate redisTemplate = SpringUtils.getBean("redisTemplate");
        redisTemplate.opsForSet().add(GrainConstant.ONLINE_USERS, userId);
    }

    /**
     * 在线用户删除
     */
    public static void subOnlineUser(Integer userId) {
        RedisTemplate redisTemplate = SpringUtils.getBean("redisTemplate");
        redisTemplate.opsForSet().remove(GrainConstant.ONLINE_USERS, userId);
    }

    /**
     * 是否在该服务上存在连接的用户
     * @param userId
     * @return
     */
    public static boolean containsUser(Integer userId) {
        return sessionPools.containsKey(userId);
    }

}
