package com.unissoft.job;

import com.alibaba.fastjson.JSONObject;
import com.unissoft.constant.GrainConstant;
import com.unissoft.dto.MessageVO;
import com.unissoft.dto.ProcessToBeDone;
import com.unissoft.dto.ToBeDoneCallbackVO;
import com.unissoft.enums.MessageTypeEnum;
import com.unissoft.feign.BusinessFeign;
import com.unissoft.service.QuartzService;
import com.unissoft.service.SysMessageService;
import com.unissoft.utils.DateUtil;
import com.unissoft.vo.WarnScheduleVO;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@DisallowConcurrentExecution
@Component
public class ToBeDoneJob implements Job, Serializable {

    private static final String WARN_NOTICE_JOB_NAME_PREFIX = "warnNoticeJob";

    private static final String WARN_NOTICE_TRIGGER_NAME_PREFIX = "warnNoticeTrigger";

    private static final String WARN_NOTICE_GROUP = "warnNoticeGroup";

    private static final long serialVersionUID = 1L;

    @Autowired
    private SysMessageService sysMessageService;

    @Autowired
    private QuartzService quartzService;

    @Autowired
    private BusinessFeign businessFeign;

    @Override
    public void execute(JobExecutionContext context) {
        JobDetail jobDetail = context.getJobDetail();
        JobDataMap dataMap = jobDetail.getJobDataMap();
        String title = dataMap.getString("title");
        String usersInfo = dataMap.getString("usersInfo");
        String timePeriod = dataMap.getString("timePeriod");
        String aheadTime = dataMap.getString("aheadTime");//预警时间
        Integer levelDepotHouse = dataMap.getInt("levelDepotHouse");
        String toPage = dataMap.getString("toPage");
        String type = dataMap.getString("type");
        //
        ProcessToBeDone done = new ProcessToBeDone();
        done.setTitle(title);
        done.setUsersInfo(usersInfo);
        Date startDate = getStartDate(timePeriod);
        done.setStartDate(startDate);
        Date endDate = getEndDate(startDate, timePeriod);
        done.setEndDate(endDate);
        done.setLevelDepotHouse(levelDepotHouse);
        done.setToPage(toPage);
        done.setType(type);
        //生成待办数据
        List<ToBeDoneCallbackVO> toBeDoneCallbackList = businessFeign.toBeDone(done);

        ////////////////////////////////////////////////////////////////////////
        if(null != toBeDoneCallbackList && toBeDoneCallbackList.isEmpty() == false) {
            //消息推送
            for(ToBeDoneCallbackVO vo : toBeDoneCallbackList) {
                MessageVO messageVO = new MessageVO();
                messageVO.setTitle(title);
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.append(vo.getDepotName()).append("-");
                if(StringUtils.isNotBlank(vo.getHouseName())) {
                    stringBuilder.append(vo.getHouseName()).append("-");
                }
                stringBuilder.append("待处理");

                String content = stringBuilder.toString();
                messageVO.setContent(content);
                //
                messageVO.setFromUserId(-1);//-1 代表系统推送
                messageVO.setToUserId(vo.getUserId());
                messageVO.setType(MessageTypeEnum.ATTENTION.getCode());
                sysMessageService.sendMessage(messageVO);
            }

            //预警通知调度
            Date warnDate = getWarnDate(endDate, aheadTime);
            long currentTimeMillis = System.currentTimeMillis();
            String jobName = WARN_NOTICE_JOB_NAME_PREFIX + "_" + currentTimeMillis;
            String triggerName = WARN_NOTICE_TRIGGER_NAME_PREFIX + "_" + currentTimeMillis;
            String description = title;
            String cronExpression = DateUtil.getCron(warnDate);
            Map<String, Object> warnNoticeDataMap = new HashMap<>();
            warnNoticeDataMap.put("warnNotices", JSONObject.toJSONString(toBeDoneCallbackList));
            warnNoticeDataMap.put("title", title);

            //
            WarnScheduleVO warnScheduleVO = new WarnScheduleVO();
            warnScheduleVO.setJobClass(ToBeDoneNoticeWarnJob.class);
            warnScheduleVO.setJobName(jobName);
            warnScheduleVO.setGroup(WARN_NOTICE_GROUP);
            warnScheduleVO.setTriggerName(triggerName);
            warnScheduleVO.setDescription(description);
            warnScheduleVO.setCronExpression(cronExpression);
            warnScheduleVO.setDataMap(warnNoticeDataMap);

            quartzService.warnScheduleJob(warnScheduleVO);
        }
        //
    }

    private Date getStartDate(String timePeriod) {
        String timeType = timePeriod.split("_")[1];

        Date startDate = null;
        if(timeType.equals(GrainConstant.STR_MINUTE)) {
            startDate = DateUtil.getCurrMinute();
        } else if(timeType.equals(GrainConstant.STR_HOUR)) {
            startDate = DateUtil.getCurrHour();
        } else if(timeType.equals(GrainConstant.STR_DAY) || timeType.equals(GrainConstant.STR_WEEKDAY)) {
            startDate = DateUtil.getCurrDate();
        } else if(timeType.equals(GrainConstant.STR_WEEK)) {
            startDate = DateUtil.getCurrDate();
        } else if(timeType.equals(GrainConstant.STR_MONTH)) {
            startDate = DateUtil.getCurrDate();
        } else if(timeType.equals(GrainConstant.STR_SEASON)) {
            startDate = DateUtil.getCurrDate();
        } else if(timeType.equals(GrainConstant.STR_YEAR)) {
            startDate = DateUtil.getCurrDate();
        }

        return startDate;
    }

    private Date getEndDate(Date startDate, String timePeriod) {
        String timeType = timePeriod.split("_")[1];
        int timeNum = Integer.parseInt(timePeriod.split("_")[0]);
        return computeDate(startDate, timeType, timeNum);
    }

    private Date computeDate(Date baseDate, String timeType, int timeNum) {
        Date date = null;
        if(timeType.equals(GrainConstant.STR_MINUTE)) {
            date = DateUtils.addMinutes(baseDate, timeNum);
        } else if(timeType.equals(GrainConstant.STR_HOUR)) {
            date = DateUtils.addHours(baseDate, timeNum);
        } else if(timeType.equals(GrainConstant.STR_DAY) || timeType.equals(GrainConstant.STR_WEEKDAY)) {
            date = DateUtils.addDays(baseDate, timeNum);
        } else if(timeType.equals(GrainConstant.STR_WEEK)) {
            date = DateUtils.addWeeks(baseDate, timeNum);
        } else if(timeType.equals(GrainConstant.STR_MONTH)) {
            date = DateUtils.addMonths(baseDate, timeNum);
        } else if(timeType.equals(GrainConstant.STR_SEASON)) {
            date = DateUtils.addMonths(baseDate, 3 * timeNum);
        } else if(timeType.equals(GrainConstant.STR_YEAR)) {
            date = DateUtils.addYears(baseDate, timeNum);
        }
        return date;
    }

    private Date getWarnDate(Date endDate, String aheadTime) {
        String timeType = aheadTime.split("_")[1];
        int timeNum = Integer.parseInt(aheadTime.split("_")[0]);
        Date warnDate = computeDate(endDate, timeType, 0 - timeNum);

        //
        if(timeType.equals(GrainConstant.STR_DAY) || timeType.equals(GrainConstant.STR_WEEKDAY) || timeType.equals(GrainConstant.STR_WEEK)
                || timeType.equals(GrainConstant.STR_MONTH) || timeType.equals(GrainConstant.STR_SEASON) || timeType.equals(GrainConstant.STR_YEAR)) {
            //如果检查周期是 天、星期、月、季、年为单位，则计算为当天的工作时间8点整
            warnDate = DateUtil.getSpecificDate(warnDate,8, 0, 0);
        }
        return warnDate;
    }


}
