package com.unissoft.service.impl;

import cn.jiguang.common.ClientConfig;
import cn.jiguang.common.ServiceHelper;
import cn.jiguang.common.connection.NettyHttpClient;
import cn.jiguang.common.resp.APIConnectionException;
import cn.jiguang.common.resp.APIRequestException;
import cn.jiguang.common.resp.ResponseWrapper;
import cn.jpush.api.JPushClient;
import cn.jpush.api.push.PushResult;
import cn.jpush.api.push.model.Message;
import cn.jpush.api.push.model.Options;
import cn.jpush.api.push.model.Platform;
import cn.jpush.api.push.model.PushPayload;
import cn.jpush.api.push.model.audience.Audience;
import cn.jpush.api.push.model.notification.AndroidNotification;
import cn.jpush.api.push.model.notification.IosNotification;
import cn.jpush.api.push.model.notification.Notification;
import cn.jpush.api.report.ReceivedsResult;
import com.unissoft.config.JpushConfig;
import com.unissoft.service.JpushService;
import io.netty.handler.codec.http.HttpMethod;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Service
@Slf4j
public class JpushServiceImpl implements JpushService {

    @Resource
    JpushConfig jpushConfig;// 注入配置信息

    /**
     * 推送自定义消息，不创建通知栏提醒，由APP端拦截信息后再决定是否创建通知
     *
     * @param title     App通知栏标题
     * @param content   App通知栏内容（为了单行显示全，尽量保持在22个汉字以下）
     * @param extras    额外推送信息（不会显示在通知栏，传递数据用）
     * @param registrationIds   手机注册ID，设定哪些用户手机能接收信息
     */
    @Override
    public PushResult sendMessagePush(String title, String content, Map<String, String> extras, List<String> registrationIds) {
        ClientConfig clientConfig = getClientConfig();

        // 使用NativeHttpClient网络客户端，连接网络的方式，不提供回调函数
        JPushClient jpushClient = new JPushClient(jpushConfig.getMasterSecret(), jpushConfig.getAppkey(), null, clientConfig);
        //
        PushPayload payload = buildMessagePushPayload(title, content, extras, registrationIds);
        PushResult result = null;
        try {
            result = jpushClient.sendPush(payload);
            log.info("极光推送结果 - " + result+"，接收推送的注册ID列表:" + String.join(",", registrationIds));
        } catch (APIConnectionException e) {
            log.error("极光推送连接错误，请稍后重试 ", e);
            log.error("Sendno: " + payload.getSendno());
        } catch (APIRequestException e) {
            log.error("极光服务器响应出错，请修复！ ", e);
            log.info("HTTP Status: " + e.getStatus());
            log.info("Error Code: " + e.getErrorCode());
            log.info("Error Message: " + e.getErrorMessage());
            log.info("Msg ID: " + e.getMsgId());
            log.info("以下存在不能识别的注册ID: " + String.join(",", registrationIds));
            log.error("Sendno: " + payload.getSendno());
        }
        return result;
    }

    /**
     * 原生方式推送
     *
     * @param title     App通知栏标题
     * @param content   App通知栏内容（为了单行显示全，尽量保持在22个汉字以下）
     * @param extras    额外推送信息（不会显示在通知栏，传递数据用）
     * @param registrationIds   手机注册ID，设定哪些用户手机能接收通知
     *
     */
    @Override
    public PushResult sendNotificationPush(String title, String content, Map<String, String> extras, List<String> registrationIds) {
        ClientConfig clientConfig = getClientConfig();

        // 使用NativeHttpClient网络客户端，连接网络的方式，不提供回调函数
        JPushClient jpushClient = new JPushClient(jpushConfig.getMasterSecret(), jpushConfig.getAppkey(), null, clientConfig);
        // 设置推送方式
        PushPayload payload = buildNotificationPushPayload(title, content, extras, registrationIds);
        PushResult result = null;
        try {
            result = jpushClient.sendPush(payload);
            log.info("极光推送结果 - " + result);
        } catch (APIConnectionException e) {
            log.error("极光推送连接错误，请稍后重试 ", e);
            log.error("Sendno: " + payload.getSendno());
        } catch (APIRequestException e) {
            log.error("极光服务器响应出错，请修复！ ", e);
            log.info("HTTP Status: " + e.getStatus());
            log.info("Error Code: " + e.getErrorCode());
            log.info("Error Message: " + e.getErrorMessage());
            log.info("Msg ID: " + e.getMsgId());
            log.info("以下存在不能识别的注册ID: " + String.join(",", registrationIds));
            log.error("Sendno: " + payload.getSendno());
        }
        return result;
    }

    /**
     * 异步请求推送方式，有回调
     *
     * @param title     通知栏标题
     * @param content   通知栏内容（为了单行显示全，尽量保持在22个汉字以下）
     * @param extras    额外推送信息（不会显示在通知栏，传递数据用）
     * @param registrationIds   手机注册ID，设定哪些用户手机能接收通知
     *
     * @see "使用NettyHttpClient,异步接口发送请求"，通过回调函数可以获取推送成功与否情况
     */
    @Override
    public void sendNotificationPushWithCallback(String title, String content, Map<String, String> extras, List<String> registrationIds) {
        ClientConfig clientConfig = getClientConfig();
        String host = (String) clientConfig.get(ClientConfig.PUSH_HOST_NAME);
        final NettyHttpClient client = new NettyHttpClient(
                ServiceHelper.getBasicAuthorization(jpushConfig.getAppkey(), jpushConfig.getMasterSecret()), null,
                clientConfig);
        try {
            URI uri = new URI(host + clientConfig.get(ClientConfig.PUSH_PATH));
            PushPayload payload = buildNotificationPushPayload(title, content, extras, registrationIds);
            client.sendRequest(HttpMethod.POST, payload.toString(), uri, new NettyHttpClient.BaseCallback() {
                @Override
                public void onSucceed(ResponseWrapper responseWrapper) {
                    if (200 == responseWrapper.responseCode) {
                        log.info("极光推送成功");
                    } else {
                        log.info("极光推送失败，返回结果: " + responseWrapper.responseContent);
                    }
                }
            });
        } catch (URISyntaxException e) {
            log.error(e.getMessage());
        } finally {
            // 需要手动关闭Netty请求进程,否则会一直保留
            client.close();
        }

    }

    private ClientConfig getClientConfig() {
        ClientConfig clientConfig = ClientConfig.getInstance();
        clientConfig.setTimeToLive(Long.valueOf(jpushConfig.getLiveTime()));
        return clientConfig;
    }


    /**
     * 构建Android和IOS的推送通知对象
     */
    private PushPayload buildNotificationPushPayload(String title, String content, Map<String, String> extras, List<String> registrationIds) {
        if (extras == null || extras.isEmpty()) {
            extras = new HashMap<>();
        }
        return PushPayload.newBuilder().setPlatform(Platform.android_ios())
                .setAudience(Audience.registrationId(registrationIds))
                .setNotification(Notification.newBuilder().setAlert(content)
                        .addPlatformNotification(AndroidNotification.newBuilder().setTitle(title).addExtras(extras).build())
                        .addPlatformNotification(IosNotification.newBuilder().incrBadge(1).addExtras(extras).build())
                        .build())
                .setOptions(Options.newBuilder().setApnsProduction(true).build())
                .build();
    }

    /**
     * 构建Android和IOS的自定义消息的推送消息对象
     */
    private PushPayload buildMessagePushPayload(String title, String content, Map<String, String> extras, List<String> registrationIds) {
        return PushPayload.newBuilder().setPlatform(Platform.android_ios())
                .setAudience(Audience.registrationId(registrationIds))
                .setMessage(Message.newBuilder().setTitle(title).setMsgContent(content).addExtras(extras).build())
                .setOptions(Options.newBuilder().setApnsProduction(true).build())
                .build();
    }

    /**
     * 查询记录推送成功条数（暂未使用）
     * @param msgId 在推送返回结果PushResult中保存
     */
    public void countPush(String msgId) {
        JPushClient jpushClient = new JPushClient(jpushConfig.getMasterSecret(), jpushConfig.getAppkey());
        try {
            ReceivedsResult result = jpushClient.getReportReceiveds(msgId);
            ReceivedsResult.Received received = result.received_list.get(0);
            log.debug("Android接受信息:" + received.android_received + "\n IOS端接受信息:" + received.ios_apns_sent);
            log.debug("极光推送返回结果 - " + result);
        } catch (APIConnectionException e) {
            log.error("极光推送连接错误，请稍后重试", e);
        } catch (APIRequestException e) {
            log.error("检查错误，并修复推送请求", e);
            log.info("HTTP Status: " + e.getStatus());
            log.info("Error Code: " + e.getErrorCode());
            log.info("Error Message: " + e.getErrorMessage());
        }
    }

}
