package com.unissoft.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.constant.GrainConstant;
import com.unissoft.mapper.QuartzMapper;
import com.unissoft.model.QuartzModel;
import com.unissoft.result.PageParam;
import com.unissoft.service.QuartzService;
import com.unissoft.vo.WarnScheduleVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.quartz.*;
import org.quartz.impl.calendar.WeeklyCalendar;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class QuartzServiceImpl implements QuartzService {

    private static final String TRIGGER_NAME_PREFIX = "trigger";
    private static final String STR_JOB_METHOD_NAME = "jobMethodName";

    @Autowired
    private QuartzMapper quartzMapper;

    @Autowired
    private Scheduler scheduler;

    @PostConstruct
    public void startScheduler() {
        try {
            scheduler.start();
            //
            Calendar weekly = scheduler.getCalendar("weekly");
            if(null == weekly) {
                WeeklyCalendar weeklyCalendar = new WeeklyCalendar();
                scheduler.addCalendar("weekly", weeklyCalendar, false, false);
            }
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
    }

    /**
     * 预警调度
     * @return
     */
    @Override
    public boolean warnScheduleJob(WarnScheduleVO warnScheduleVO) {
        try {
            Class jobCls = warnScheduleVO.getJobClass();
            String jobName = warnScheduleVO.getJobName();
            String group = warnScheduleVO.getGroup();
            String triggerName = warnScheduleVO.getTriggerName();
            String description = warnScheduleVO.getDescription();
            String cronExpression = warnScheduleVO.getCronExpression();
            //
            Map<String, Object> dataMap = warnScheduleVO.getDataMap();
            dataMap.put(GrainConstant.STR_JOBNAME, jobName);
            dataMap.put("group", group);
            dataMap.put("triggerName", triggerName);

            //构建job信息
            JobDetail job = JobBuilder.newJob(jobCls).withIdentity(jobName, group).withDescription(description).build();
            job.getJobDataMap().putAll(dataMap);

            // 触发时间点
            CronScheduleBuilder cronScheduleBuilder = CronScheduleBuilder.cronSchedule(cronExpression);
            Trigger trigger = TriggerBuilder.newTrigger().withIdentity(triggerName, group).withSchedule(cronScheduleBuilder).build();
            //交由Scheduler安排触发
            scheduler.scheduleJob(job, trigger);
        } catch (Exception e) {
            log.error(e.getMessage());
            return false;
        }
        return true;
    }

    /**
     * 新增任务
     * @param quartz
     * @return
     */
    @Override
    public boolean save(QuartzModel quartz) {
        try {
            if(quartz.getOldJobGroup()!=null) {
                JobKey key = new JobKey(quartz.getOldJobName(), quartz.getOldJobGroup());
                scheduler.deleteJob(key);
            }
            //
            Class cls = Class.forName(quartz.getJobClassName()) ;
            cls.newInstance();
            String jobName = quartz.getJobName();
            String group = quartz.getJobGroup();
            String triggerName = TRIGGER_NAME_PREFIX + quartz.getJobName();

            //构建job信息
            JobDetail job = JobBuilder.newJob(cls).withIdentity(jobName, group).withDescription(quartz.getDescription()).build();
            job.getJobDataMap().put(STR_JOB_METHOD_NAME, quartz.getJobMethodName());
            // 触发时间点
            CronScheduleBuilder cronScheduleBuilder = CronScheduleBuilder.cronSchedule(quartz.getCronExpression());
            Trigger trigger = TriggerBuilder.newTrigger().withIdentity(triggerName, group).withSchedule(cronScheduleBuilder).build();
            //交由Scheduler安排触发
            scheduler.scheduleJob(job, trigger);
        } catch (Exception e) {
            log.error(e.getMessage());
            return false;
        }
        return true;
    }

    /**
     * 任务列表分页条件查询
     * @param pageParam
     * @return
     * @throws SchedulerException
     */
    @Override
    public Page<QuartzModel> getPage(PageParam pageParam) throws SchedulerException {

        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        Map<String, Object> map = Maps.newHashMap();
        if (! StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            if (object.has(GrainConstant.STR_JOBNAME)) {
                String jobName = object.getString(GrainConstant.STR_JOBNAME);
                if (! StringUtils.isEmpty(jobName)) {
                    map.put(GrainConstant.STR_JOBNAME, jobName.trim());
                }
            }
        }

        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<QuartzModel> pageResult = quartzMapper.getPage(page, map);

        List<QuartzModel> list = pageResult.getRecords();
        for (QuartzModel quartzModel : list) {
            JobKey key = new JobKey(quartzModel.getJobName(), quartzModel.getJobGroup());
            JobDetail jobDetail = scheduler.getJobDetail(key);
            quartzModel.setJobMethodName(jobDetail.getJobDataMap().getString(STR_JOB_METHOD_NAME));
        }

        return pageResult;
    }

    /**
     * 立即执行job
     * @param quartz
     * @return
     */
    @Override
    public boolean triggerJob(QuartzModel quartz) {
        try {
            JobKey key = new JobKey(quartz.getJobName(), quartz.getJobGroup());
            scheduler.triggerJob(key);
        } catch (SchedulerException e) {
            log.error(e.getMessage());
            return false;
        }
        return true;
    }

    /**
     * 停止任务
     * @param quartz
     * @return
     */
    @Override
    public boolean pauseJob(QuartzModel quartz) {
        try {
            JobKey key = new JobKey(quartz.getJobName(), quartz.getJobGroup());
            scheduler.pauseJob(key);
        } catch (SchedulerException e) {
            log.error(e.getMessage());
            return false;
        }
        return true;
    }

    /**
     * 恢复任务
     * @param quartz
     * @return
     */
    @Override
    public boolean resumeJob(QuartzModel quartz) {
        try {
            JobKey key = new JobKey(quartz.getJobName(), quartz.getJobGroup());
            scheduler.resumeJob(key);
        } catch (SchedulerException e) {
            log.error(e.getMessage());
            return false;
        }
        return true;
    }

    /**
     * 删除任务
     * @param quartz
     * @return
     */
    @Override
    public boolean removeJob(QuartzModel quartz) {
        String jobName = quartz.getJobName();
        String group = quartz.getJobGroup();
        String triggerName = TRIGGER_NAME_PREFIX + quartz.getJobName();

        return removeJob(jobName, triggerName, group);
    }

    /**
     * 删除任务
     * @param jobName
     * @param group
     * @param triggerName
     * @return
     */
    @Override
    public boolean removeJob(String jobName, String triggerName, String group) {
        try {
            TriggerKey triggerKey = TriggerKey.triggerKey(triggerName, group);
            scheduler.pauseTrigger(triggerKey);// 停止触发器
            scheduler.unscheduleJob(triggerKey);// 移除触发器
            scheduler.deleteJob(JobKey.jobKey(jobName, group));// 删除任务
        } catch (Exception e) {
            log.error(e.getMessage());
            return false;
        }
        return true;
    }

}


